<?php

namespace App\Controllers;

use App\Models\HotelModel;
use App\Models\CityModel;
use App\Models\RoomModel;
use App\Models\RoomTypeModel;
use CodeIgniter\API\ResponseTrait;

class HotelManagement extends BaseController
{
    use ResponseTrait;

    protected $hotelModel;
    protected $cityModel;
    protected $roomModel;

    public function __construct()
    {
        $this->hotelModel = new HotelModel();
        $this->cityModel = new CityModel();
        $this->roomModel = new RoomModel();
      	$this->roomTypeModel = new RoomTypeModel();
    }

    public function index()
    {
        $data['hotels'] = $this->hotelModel->findAll();
        $data['cities'] = $this->cityModel->findAll();
        $data['rooms'] = $this->roomModel->findAll();
      	$data['roomTypes'] = $this->roomTypeModel->findAll();
        
        return view('hotel_management/index', $data);
    }

    public function add()
    {
        try {
            $data = $this->request->getPost();
            $data['is_active'] = isset($data['is_active']) ? 1 : 0;

            if ($this->hotelModel->insert($data)) {
                $hotelId = $this->hotelModel->getInsertID();
                
                $roomTypeIds = $this->request->getPost('room_types');
                if (!empty($roomTypeIds)) {
                    foreach ($roomTypeIds as $roomTypeId) {
                        $this->roomModel->insert([
                            'hotel_id' => $hotelId,
                            'room_type_id' => $roomTypeId
                        ]);
                    }
                }

                return $this->respond(['success' => true, 'message' => 'Hotel added successfully.'], 200);
            } else {
                $errors = $this->hotelModel->errors();
                log_message('error', 'Failed to add hotel: ' . json_encode($errors));
                return $this->respond(['success' => false, 'message' => 'Failed to add hotel.', 'errors' => $errors], 400);
            }
        } catch (\Exception $e) {
            log_message('error', 'Exception when adding hotel: ' . $e->getMessage());
            return $this->respond(['success' => false, 'message' => 'An error occurred while adding the hotel.'], 500);
        }
    }

    public function edit($id)
	{
    $hotel = $this->hotelModel->find($id);
    if ($hotel) {
        // Get all room types associated with this hotel
        $hotelRoomTypes = $this->roomModel->select('room_type_id')
                                          ->where('hotel_id', $id)
                                          ->findAll();
        
        $hotel['room_types'] = array_column($hotelRoomTypes, 'room_type_id');

        // Get all available room types
        $allRoomTypes = $this->roomTypeModel->findAll();
        
        $data = [
            'success' => true,
            'hotel' => $hotel,
            'allRoomTypes' => $allRoomTypes
        ];
        
        return $this->respond($data, 200);
    } else {
        return $this->respond(['success' => false, 'message' => 'Hotel not found.'], 404);
    }
	}

    public function update($id)
{
    try {
        $data = $this->request->getPost();
        $data['is_active'] = isset($data['is_active']) ? 1 : 0;

        if ($this->hotelModel->update($id, $data)) {
            // Get current room types for this hotel
            $currentRoomTypes = $this->roomModel->where('hotel_id', $id)->findAll();
            $currentRoomTypeIds = array_column($currentRoomTypes, 'room_type_id');

            // Get new room types from the form
            $newRoomTypeIds = $this->request->getPost('room_types') ?? [];

            // Room types to add
            $roomTypesToAdd = array_diff($newRoomTypeIds, $currentRoomTypeIds);

            // Room types to remove
            $roomTypesToRemove = array_diff($currentRoomTypeIds, $newRoomTypeIds);

            // Add new room types
            foreach ($roomTypesToAdd as $roomTypeId) {
                $this->roomModel->insert([
                    'hotel_id' => $id,
                    'room_type_id' => $roomTypeId
                ]);
            }

            // Remove room types
            foreach ($roomTypesToRemove as $roomTypeId) {
                // Check if this room is referenced in request_hotels
                $referencedRoom = $this->roomModel->where('hotel_id', $id)
                                                  ->where('room_type_id', $roomTypeId)
                                                  ->first();
                
                if ($referencedRoom) {
                    $isReferenced = $this->db->table('request_hotels')
                                             ->where('room_id', $referencedRoom['id'])
                                             ->countAllResults() > 0;

                    if ($isReferenced) {
                        // If referenced, don't delete but maybe mark as inactive or handle differently
                        $this->roomModel->update($referencedRoom['id'], ['is_active' => 0]);
                    } else {
                        // If not referenced, safe to delete
                        $this->roomModel->delete($referencedRoom['id']);
                    }
                }
            }

            return $this->respond(['success' => true, 'message' => 'Hotel updated successfully.'], 200);
        } else {
            $errors = $this->hotelModel->errors();
            log_message('error', 'Failed to update hotel: ' . json_encode($errors));
            return $this->respond(['success' => false, 'message' => 'Failed to update hotel.', 'errors' => $errors], 400);
        }
    } catch (\Exception $e) {
        log_message('error', 'Exception when updating hotel: ' . $e->getMessage() . "\nTrace: " . $e->getTraceAsString());
        return $this->respond(['success' => false, 'message' => 'An error occurred while updating the hotel: ' . $e->getMessage()], 500);
    }
}

    public function delete($id)
    {
        try {
            $this->roomModel->where('hotel_id', $id)->delete();
            if ($this->hotelModel->delete($id)) {
                return $this->respond(['success' => true, 'message' => 'Hotel deleted successfully.'], 200);
            } else {
                return $this->respond(['success' => false, 'message' => 'Failed to delete hotel.'], 400);
            }
        } catch (\Exception $e) {
            log_message('error', 'Exception when deleting hotel: ' . $e->getMessage());
            return $this->respond(['success' => false, 'message' => 'An error occurred while deleting the hotel.'], 500);
        }
    }

    public function toggleActive($id)
    {
        try {
            $hotel = $this->hotelModel->find($id);
            if ($hotel) {
                $hotel['is_active'] = $hotel['is_active'] ? 0 : 1;
                if ($this->hotelModel->update($id, $hotel)) {
                    return $this->respond(['success' => true, 'message' => 'Hotel status updated successfully.'], 200);
                } else {
                    return $this->respond(['success' => false, 'message' => 'Failed to update hotel status.'], 400);
                }
            } else {
                return $this->respond(['success' => false, 'message' => 'Hotel not found.'], 404);
            }
        } catch (\Exception $e) {
            log_message('error', 'Exception when toggling hotel status: ' . $e->getMessage());
            return $this->respond(['success' => false, 'message' => 'An error occurred while updating the hotel status.'], 500);
        }
    }
}