<?php

namespace App\Models;

use CodeIgniter\Model;

class RtqModel extends Model
{
    protected $table = 'rtq';
    protected $primaryKey = 'id';
    protected $allowedFields = ['request_id', 'hotel_id', 'status', 'notes', 'voucher_path'];
    protected $useTimestamps = true;
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    public function getAllRtqs()
    {
        return $this->select('rtq.*, requests.unique_id as request_unique_id')
                    ->join('requests', 'requests.id = rtq.request_id')
                    ->orderBy('rtq.created_at', 'DESC')
                    ->findAll();
    }

    public function getRtqDetails($id)
    {
        $rtq = $this->select('rtq.*, requests.unique_id, requests.status as request_status, requests.created_at as request_created_at, 
                              users.username, 
                              f1.flight_number as arrival_flight_number, 
                              f2.flight_number as departure_flight_number')
                    ->join('requests', 'requests.id = rtq.request_id')
                    ->join('users', 'users.id = requests.user_id')
                    ->join('flights as f1', 'f1.id = requests.arrival_flight_id', 'left')
                    ->join('flights as f2', 'f2.id = requests.departure_flight_id', 'left')
                    ->where('rtq.id', $id)
                    ->first();

        if ($rtq) {
            $rtq['hotels'] = $this->getHotelsForRtq($rtq['request_id']);
            $rtq['passengers'] = $this->getPassengersForRtq($rtq['request_id']);
        }

        return $rtq;
    }
  
  	public function updateHotelStatus($rtqId, $hotelId, $status)
    {
        try {
            // First, get the request_id associated with this RTQ
            $rtq = $this->find($rtqId);
            if (!$rtq) {
                log_message('warning', "No RTQ found for ID {$rtqId}");
                return false;
            }

            $requestId = $rtq['request_id'];

            // Now, check if the row exists in request_hotels
            $existingRow = $this->db->table('request_hotels')
                ->where('request_id', $requestId)
                ->where('hotel_id', $hotelId)
                ->get()
                ->getRow();

            if (!$existingRow) {
                log_message('warning', "No matching row found in request_hotels for Request ID {$requestId} and Hotel ID {$hotelId}");
                return false;
            }

            // Update the status
            $result = $this->db->table('request_hotels')
                ->where('request_id', $requestId)
                ->where('hotel_id', $hotelId)
                ->update(['rtq_status' => $status]);

            if ($this->db->affectedRows() > 0) {
                log_message('info', "Successfully updated hotel status: RTQ ID {$rtqId}, Request ID {$requestId}, Hotel ID {$hotelId}, Status {$status}");
                return true;
            } else {
                log_message('warning', "Update query executed but no rows affected: RTQ ID {$rtqId}, Request ID {$requestId}, Hotel ID {$hotelId}, Status {$status}");
                log_message('info', "Current status in database: " . ($existingRow->rtq_status ?? 'N/A'));
                return false;
            }
        } catch (\Exception $e) {
            log_message('error', "Error updating hotel status: " . $e->getMessage());
            return false;
        }
    }
  
  	public function getHotelsForRtq($requestId)
	{
    return $this->db->table('request_hotels')
        ->select('request_hotels.*, hotels.name as hotel_name, hotels.email as hotel_email, room_types.name as room_type')
        ->join('hotels', 'hotels.id = request_hotels.hotel_id')
        ->join('rooms', 'rooms.id = request_hotels.room_id')
        ->join('room_types', 'room_types.id = rooms.room_type_id')
        ->where('request_hotels.request_id', $requestId)
        ->get()
        ->getResultArray();
	}
  
  	public function getPassengersForRtq($requestId)
    {
        return $this->db->table('passengers')
            ->where('request_id', $requestId)
            ->get()
            ->getResultArray();
    }
  
  public function getVoucherCount($rtqId, $hotelId)
    {
        $result = $this->db->table('rtq_vouchers')
            ->where('rtq_id', $rtqId)
            ->where('hotel_id', $hotelId)
            ->get()
            ->getRowArray();

        return $result ? $result['voucher_count'] : 0;
    }
  
  	public function incrementVoucherCount($rtqId, $hotelId)
    {
        $existingCount = $this->getVoucherCount($rtqId, $hotelId);

        if ($existingCount > 0) {
            $this->db->table('rtq_vouchers')
                ->where('rtq_id', $rtqId)
                ->where('hotel_id', $hotelId)
                ->update(['voucher_count' => $existingCount + 1]);
        } else {
            $this->db->table('rtq_vouchers')->insert([
                'rtq_id' => $rtqId,
                'hotel_id' => $hotelId,
                'voucher_count' => 1
            ]);
        }
    }

    
}